package com.fjhx.xmhjc.service.column.impl;

import com.fjhx.xmhjc.entity.column.po.ColumnMenuSub;
import com.fjhx.xmhjc.mapper.column.ColumnMenuSubMapper;
import com.fjhx.xmhjc.service.column.ColumnMenuSubService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.column.vo.ColumnMenuSubVo;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSubSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.column.dto.ColumnMenuSubDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 栏目-子栏目 服务实现类
 * </p>
 *
 * @author lqh
 * @since 2023-11-15
 */
@Service
public class ColumnMenuSubServiceImpl extends ServiceImpl<ColumnMenuSubMapper, ColumnMenuSub> implements ColumnMenuSubService {

    @Override
    public List<ColumnMenuSubVo> getList(ColumnMenuSubSelectDto dto) {
        IWrapper<ColumnMenuSub> wrapper = getWrapper();
        wrapper.eq("cms", ColumnMenuSub::getColumnId,dto.getColumnId());
        wrapper.orderByAsc("cms", ColumnMenuSub::getSort);
        List<ColumnMenuSubVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<ColumnMenuSubVo> getPage(ColumnMenuSubSelectDto dto) {
        IWrapper<ColumnMenuSub> wrapper = getWrapper();
        wrapper.orderByDesc("cms", ColumnMenuSub::getId);
        Page<ColumnMenuSubVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ColumnMenuSubVo detail(Long id) {
        ColumnMenuSub ColumnMenuSub = this.getById(id);
        ColumnMenuSubVo result = BeanUtil.toBean(ColumnMenuSub, ColumnMenuSubVo.class);
        return result;
    }

    @Override
    public void add(ColumnMenuSubDto columnMenuSubDto) {
        this.save(columnMenuSubDto);
    }

    @Override
    public void edit(ColumnMenuSubDto columnMenuSubDto) {
        this.updateById(columnMenuSubDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);


    }

    @Transactional
    public void handleColumnMenuSub(Long id, List<ColumnMenuSub> columnMenuSubList) {
        //获取已有子栏目
        IWrapper<ColumnMenuSub> wrapper = getWrapper();
        wrapper.eq("cms", ColumnMenuSub::getColumnId,id);
        List<ColumnMenuSubVo> list = this.baseMapper.getList(wrapper);
        //用于删除子栏目
        Map<Long, ColumnMenuSubVo> map = list.stream().collect(Collectors.toMap(ColumnMenuSub::getId, Function.identity()));

        //新增修改
        columnMenuSubList.stream().forEach(sub ->{
            sub.setColumnId(id);
            //判断新增修改
            if (Objects.isNull(sub.getId())){
               this.save(sub);
            }else {
                this.updateById(sub);
                //去除已存在Id
                map.remove(sub.getId());
            }
        });

        //删除
        map.forEach((key,value) ->{
            //TODO判断文章
            this.removeById(key);
        });

    }
}
